unit Unit1;
{*****************************************************************************}
{    F.A.Q Delphi - www.developpez.com                                        }
{                                                                             }
{ Comment ajouter une icone dans la barre des taches                          }
{ Auteur : Nono40                                                             }
{                                                                             }
{ Ralis avec Delphi 7                                                       }
{*****************************************************************************}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Menus, StdCtrls;

Const
  // Dfinition d'un message personnalis.
  // Ce message est envoy par Windows sur demande quand un vnement
  // li  la souris intervient.
  WM_MONICONE = WM_USER +1;

type
  TForm1 = class(TForm)
    TimerMiseAJour: TTimer;
    PopUpGauche: TPopupMenu;
    ActiverSuiviItem: TMenuItem;
    PopUpDroit: TPopupMenu;
    QuitterItem: TMenuItem;
    TimerRetardMessage: TTimer;
    RestaurerItem: TMenuItem;
    ReduireItem: TMenuItem;
    Label1: TLabel;
    Label2: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure TimerMiseAJourTimer(Sender: TObject);
    procedure ActiverSuiviItemClick(Sender: TObject);
    procedure QuitterItemClick(Sender: TObject);
    procedure TimerRetardMessageTimer(Sender: TObject);
    procedure RestaurerItemClick(Sender: TObject);
    procedure ReduireItemClick(Sender: TObject);
    procedure Label2MouseEnter(Sender: TObject);
    procedure Label2MouseLeave(Sender: TObject);
    procedure Label2Click(Sender: TObject);
  private
    { Dclarations prives }
    // Ce tableau regroupe est Handles des icones extraites du fichier
    LesIcones    :Array[0..5]Of HICON;
    // Numero dans le tableau ci-dessus de l'icone actuellement affiche
    IconeEnCours :Integer;
    // Mmorisation du dernier message reu par l'icone
    // ( voir son utilit dans MessageIcone() )
    DernierMSG   :TMsg;

    // Dfinition d'une mthode de rponse au message personnalis cr
    Procedure MessageIcone(Var Msg: TMsg);Message WM_MONICONE;
    // Procdure d'ajout de l'icone, est appele au dbut de l'application
    Procedure AjouteIcone;
    // Procdure de mise  jour de l'icone
    Procedure ModifieIcone;
    // Procdure de suppression de l'icone, est appele  la fermeture
    Procedure SupprimeIcone;
  public
    { Dclarations publiques }
  end;

var
  Form1: TForm1;

implementation

Uses ShellApi;
{$R *.dfm}

// Ajout d'un fichier de ressource contenant les icones affiches dans la barre des taches
// Les icones tant dfinies par un numro de 0  n dans le fichier exe en fonction
// de leur ordre alphabtique ( leur nom dans le .RES ).
// le mieux est de dfinir des noms d'icones en Zxxxx afin que l'icone de l'exe soit celle
// contenu dans les options de projet.
{$R icones.res}

// Au dmarrage de l'application
procedure TForm1.FormCreate(Sender: TObject);
Var i:Integer;
begin
  // On extrait les icones du fichier .EXE de l'application
  For i:=0 to 5 Do LesIcones[i]:=ExtractIcon(Application.Handle,PChar(Application.exename),i);
  // L'icone est ajoute dans la barre des taches
  IconeEnCours:=0;
  AjouteIcone;
end;

// A la fin de l'application
procedure TForm1.FormDestroy(Sender: TObject);
Var i:Integer;
begin
  // L'icone est supprime de la barre des taches
  SupprimeIcone;
  // Et les handles des icones sont librs
  For i:=0 To 5 Do If LesIcones[i]>1 Then DestroyIcon(LesIcones[i]);
end;

// Procdure d'ajout d'une icone dans la barre des taches
Procedure TForm1.AjouteIcone;
Var Info:TNotifyIconData;
begin
  // If faut tout d'abord remplir la structure Info
  // avec ce que l'on veut faire

  // cbSize doit contenir la taille de la structure
  Info.cbSize := SizeOf(Info);
  // Wnd doit contenir le Handle de la fentre qui recevra les messages de
  // notification d'vnement de la souris
  Info.Wnd    := Handle;
  // uID Numro d'icone, c'est utile si plusieurs icones sont affiches en
  // simultannes dans la barre des taches. Car c'est ce numro qui permettra
  // ensuite de modifier celle que l'on veut.
  Info.uID    := 1;
  // szTip contient le texte de l'info bulle affich au dessus de l'icone
  Info.szTip  := 'Exemple d''icones par Nono40';
  // hIcon contient le handle de l'icone qui doit tre affiche
  Info.hIcon  := LesIcones[IconeEnCours];
  // uCallBackMessage contient le message qui sera retourn  la fentre
  // donne par Wnd quand un vnement de souris apparait sur l'icone
  Info.uCallbackMessage := WM_MONICONE;
  // uFlags doit contenir le liste des champs utiliss dans la structure
  // parmis les champs szTip,hIcon et uCallBackMessage
  Info.uFlags := NIF_TIP Or NIF_ICON Or NIF_MESSAGE;

  // Appel de la fonction API ajoutant l'icone
  Shell_NotifyIcon(NIM_ADD,@Info);
End;

Procedure TForm1.ModifieIcone;
Var Info:TNotifyIconData;
begin
  // If faut tout d'abord remplir la structure Info
  // avec ce que l'on veut faire
  // ( voir la procdure ci-dessus pour les dtails )
  Info.cbSize := SizeOf(Info);
  Info.Wnd    := Handle;
  Info.uID    := 1;
  Info.uFlags := NIF_TIP Or NIF_ICON Or NIF_MESSAGE;
  Info.szTip  := 'Exemple d''icones par Nono40';
  Info.hIcon  := LesIcones[IconeEnCours];
  Info.uCallbackMessage := WM_MONICONE;
  Shell_NotifyIcon(NIM_MODIFY,@Info);
End;

Procedure TForm1.SupprimeIcone;
Var Info:TNotifyIconData;
begin
  // Dans le cas de la suppression d'une icone, seuls les champs ci-dessous
  // sont ncessaires
  Info.cbSize := SizeOf(Info);
  Info.Wnd    := Handle;
  Info.uID    := 1;
  Shell_NotifyIcon(NIM_DELETE,@Info);
End;

// Procdure lie au timer charge de changer l'icone si besoin
procedure TForm1.TimerMiseAJourTimer(Sender: TObject);
Var X             :Integer;
    Y             :Integer;
    NouvelleIcone :Integer;
begin
  // En premier lieu, on dtermine l'icone qui doit tre affiche
  If ActiverSuiviItem.Checked
  Then Begin
    // Si le suivi de la souris est actif, il faut calculer l'angle
    // antre le bas de l'cran et la ligne joignant la souris au coin bas-droit
    // de l'cran. ArcTan(Y/X) retourne l'angle en Radian, la nouvelle icone
    // n'est ensuite qu'une rgle de trois pour obtenir une valeur entre 1 et 5
    // ( car c'est le nombre d'icones diffrentes )
    X := Screen.Width  +30 -Mouse.CursorPos.X;
    Y := Screen.Height +30 -Mouse.CursorPos.Y;
    NouvelleIcone:=Round(Arctan(Y/X)/(PI/2)*5)+1;
    // Il faut quand mme vrifier que le rsultat est cohrent
    // ce n'est pas forcment utile, mais bon enfin c'est l...
    If NouvelleIcone<1 Then NouvelleIcone:=1;
    If NouvelleIcone>5 Then NouvelleIcone:=5;
  End
  Else Begin
    // Si le suivi de la souris n'est pas activ
    // c'est l'icone de base qui est affiche
    NouvelleIcone:=0;
  End;
  // Dans le cas ou l'icone doit tre change, ben alors elle est change...
  If NouvelleIcone<>IconeEnCours
  Then Begin
    IconeEnCours:=NouvelleIcone;
    ModifieIcone;
  End;
end;

// Procdures du menu du bouton GAUCHE
procedure TForm1.ActiverSuiviItemClick(Sender: TObject);
begin
  // Inversion du suivi de la souris...
  ActiverSuiviItem.Checked := Not ActiverSuiviItem.Checked;
end;

// Procdures du menu du bouton DROIT
procedure TForm1.QuitterItemClick(Sender: TObject);
begin
  // Quitter l'application
  Close;
end;

procedure TForm1.RestaurerItemClick(Sender: TObject);
begin
  // Restaurer l'application
  Application.Restore;
end;

procedure TForm1.ReduireItemClick(Sender: TObject);
begin
  // Rduire l'application
  Application.Minimize;
end;

// Procdure de rponse  notre message personnalis
// A chaque vnement de la souris, cette procdure sera appele
// Windows retourne dans le champ wParam du message le message original
// de l'icone.
// Ainsi, MSG.Message contient WM_MONICONE
// et MSG.wParam contient WM_LBUTTONDOWN ou WM_LBUTTONUO ou ...
// Il est donc facile de faire le tri
Procedure TForm1.MessageIcone(Var MSG:TMSG);
Begin
  // Click du bouton gauche
  If MSG.wParam=WM_LBUTTONDOWN
  Then Begin
    // Attention, si le double click doit tre gr, il ne faut pas excuter
    // tout de suite l'action li  ce message. Surtout si un menu est affich
    // sur le click gauche. Dans ce cas le double-click ne se produit pas car
    // le deuxime click n'est plus li  l'icone mais au menu.

    // Le message est enregistr, pour tre tester plus tard si aucun autre
    // click et/ou double-clique n'est reu
    DernierMSG := MSG;
    // C'est le timer qui excutera le message en cas de besoin
    TimerRetardMessage.Enabled:=True;
  End;
  If MSG.wParam=WM_RBUTTONUP
  Then Begin
    // Dans le cas du bouton droit, le double-clique n'est pas gr
    // le menu li au bouton droit peut donc tre affich de suite.
    // Il est prfrable dans ce cas d'utiliser WM_RBUTTONUP plutt
    // que WM_RBUTTONDOWN
    PopUpDroit .Popup(Mouse.CursorPos.X,Mouse.CursorPos.Y);
  End;
  If MSG.wParam=WM_LBUTTONDBLCLK
  Then Begin
    // Un double-clique est dtect, alors il faut annuler le timer avant
    // qu'il n'arrive  terme. Ainsi le l'action lie au click simple
    // n'est pas excute
    TimerRetardMessage.Enabled:=False;
    // Le double-click ici restaure l'application
    // et elle est mise au premier plan
    Application.Restore;
    Application.BringToFront
  End;
End;

// Procdure li au timer de retard sur le click-gauche
procedure TForm1.TimerRetardMessageTimer(Sender: TObject);
begin
  // Il faut vrifier que le message en attente est bien un click-gauche
  If DernierMSG.wParam=WM_LBUTTONDOWN
  Then Begin
    // Dans ce cas le menu li au click gauche est affich
    PopUpGauche.Popup(Mouse.CursorPos.X,Mouse.CursorPos.Y);
  End;
  // Le timer est ensuite annul pour ne pas rpter le message...
  TimerRetardMessage.Enabled := False;
end;

procedure TForm1.Label2MouseEnter(Sender: TObject);
begin
  Label2.Font.Style:=[fsUnderline,fsBold];
end;

procedure TForm1.Label2MouseLeave(Sender: TObject);
begin
  Label2.Font.Style:=[fsBold];
end;

procedure TForm1.Label2Click(Sender: TObject);
begin
  ShellExecute(Handle,'OPEN','http://delphi.developpez.com',nil,nil,SW_SHOWNORMAL);
end;

end.
